#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */
#include "usbdrv.h"

#define RQ_GET_AVR2    2 // SENSOR        	PC0
#define RQ_GET_AVR3    3 // LDR ตัวที่1		PC1
#define RQ_GET_AVR4    4 // LDR ตัวที่2		PC2
#define RQ_GET_AVR5    5 // LDR ตัวที่3		PC3
#define RQ_GET_AVR6    6 // LDR ตัวที่4		PC4
#define RQ_GET_AVR7    13 // Thermistor		PC5

#define RQ_GET_SWITCH1  7 // switch ตัวที่ 1	PB3
#define RQ_SET_LED1     8 // LED ตัวที่1		PB0
#define RQ_SET_LED2     9 // LED ตัวที่2		PB1
#define RQ_SET_LED3    10 // LED ตัวที่3		PB2
#define RQ_GET_SWITCH2 11 // switch ตัวที่ 2	PB4
#define RQ_SET_LED4    12 // RING		PB5

#define RQ_SET_LED5    14 // LED ตัวที่4		PD1
#define RQ_SET_LED6    15 // MOTOR		PD6
/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */

uint16_t read_adc(uint8_t channel)
	{
    	ADMUX = 0b01000000; // เลือกใช้แรงดันอ้างอิงจากขา AREF
    	ADMUX |= channel;   // ตั้ง MUX ให้ชี้ไปยัง channel
    	ADCSRA = 0b11000110;
    	//         ||   -+-
    	//         ||    |
    	//         ||    +-> ปรับคล็อกของ ADC ให้เป็น 1/64 ของคล็อกระบบ
    	//         |+->เริ่มแปลงค่าแอนะล็อกเป็นดิจิทัล (บิต ADSC)
    	//         |
    	//         +-> Enable ADC (บิต ADEN)

    	while ((ADCSRA & (1<<ADSC))) // รอจนบิต ADSC (บิต 6) กลายเป็น 0
        	;
    	return ADCL + ADCH*256;  // ผลลัพธ์ถูกเก็บอยู่ในรีจีสเตอร์ ADCL และ ADCH
	}

usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;
    static uint8_t switch_state;  /* must stay valid when usbFunctionSetup returns */
    static uint16_t adc;


/* ------------------------------------------------------------------------- */
// PORTC
   


   if (rq->bRequest == RQ_GET_AVR2)
    {
	adc = read_adc(PC0);
	usbMsgPtr = &adc;
	return 2;
    }

    if (rq->bRequest == RQ_GET_AVR3)
    {
	adc = read_adc(PC1);
	usbMsgPtr = &adc;
	return 3;
    }

    if (rq->bRequest == RQ_GET_AVR4)
    {
	adc = read_adc(PC2);
	usbMsgPtr = &adc;
	return 4;
    }

    if (rq->bRequest == RQ_GET_AVR5)
    {
	adc = read_adc(PC3);
	usbMsgPtr = &adc;
	return 5;
    }

    if (rq->bRequest == RQ_GET_AVR6)
    {
	adc = read_adc(PC4);
	usbMsgPtr = &adc;
	return 6;
    }

     if (rq->bRequest == RQ_GET_AVR7)
    {
	adc = read_adc(PC5);
	usbMsgPtr = &adc;
	return 13;
    } 


/* ------------------------------------------------------------------------- */
//PORTB

    if (rq->bRequest == RQ_GET_SWITCH1)
    {
        if ((PINB & (1<<PB3)) == 0) /* switch is pressed */
            switch_state = 1;
        else
            switch_state = 0;

        usbMsgPtr = (uchar*) &switch_state;
        return sizeof(switch_state);
    }
  if (rq->bRequest == RQ_GET_SWITCH2)
  {
      if ((PINB & (1<<PB4)) == 0) /* switch is pressed */
          switch_state = 1;
      else
          switch_state = 0;

      usbMsgPtr = (uchar*) &switch_state;
      return sizeof(switch_state);
  }

    if (rq->bRequest == RQ_SET_LED1)
    {
        uint8_t led_val = rq->wValue.bytes[0];

        if (led_val)
            PORTB |= (1<<PB0);
        else
            PORTB &= ~(1<<PB0);
            return 8;
    }

    if (rq->bRequest == RQ_SET_LED2)
    {
        uint8_t led_val = rq->wValue.bytes[0];

        if (led_val)
            PORTB |= (1<<PB1);
        else
            PORTB &= ~(1<<PB1);
            return 9;
    }

    if (rq->bRequest == RQ_SET_LED3)
    {
        uint8_t led_val = rq->wValue.bytes[0];

        if (led_val)
            PORTB |= (1<<PB2);
        else
            PORTB &= ~(1<<PB2);
            return 10;
    }

    if (rq->bRequest == RQ_SET_LED4)
    {
        uint8_t led_val = rq->wValue.bytes[0];

        if (led_val)
            PORTB |= (1<<PB5);
        else
            PORTB &= ~(1<<PB5);
            return 12;
    }
   

/* ------------------------------------------------------------------------- */
//PORTD
    if (rq->bRequest == RQ_SET_LED5)
    {
        uint8_t led_val = rq->wValue.bytes[0];

        if (led_val)
            PORTD |= (1<<PD0);
        else
            PORTD &= ~(1<<PD0);
            return 14;
    }
    if (rq->bRequest == RQ_SET_LED6)
    {
        uint8_t led_val = rq->wValue.bytes[0];

        if (led_val)
            PORTD |= (1<<PD1);
        else
            PORTD &= ~(1<<PD1);
            return 15;
    }



return 0;   /* default for not implemented requests: return no data back to host */
}

/* ------------------------------------------------------------------------- */

int main(void)
{
   //DDRC  = (1<<PC0)|(1<<PC1)|(1<<PC2); // PC0..PC2 = Output, PC3..PC5 = Input
    //PORTC = uint8_t count = 0; //เก็บค่าได้ 0...255
        PORTC |= (1 << PC1)|(1 << PC2)|(1 << PC3)|(1 << PC4)|(1 << PC5); 

	PORTB |= (1 << PB3) |(1 << PB4);
	DDRB |= (1 << PB0)|(1 << PB1) |(1 << PB2) | (1 << PB5);	

	DDRD |= (1 << PD0)|(1 << PD1);	


    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
        usbPoll();
    }

    return 0;
}

/* ------------------------------------------------------------------------- */
